#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>


static const int skDataLen = 4096;
static const int skTruncatedLen = 3766;
	// just anything smaller than skDataLen, to leave a gap in the last page
static char testData[skDataLen];


int
main()
{
	int fileDes = open("mmap_zero_test.data", O_CREAT | O_RDWR | O_TRUNC);
	if (fileDes == -1) {
		fprintf(stderr, "unable to create test-file - errno=%x\n", errno);
		return 1;
	}

	memset(testData, 'x', skDataLen);

	ssize_t bytesWritten = write(fileDes, testData, skDataLen);
	if (bytesWritten < 0) {
		fprintf(stderr, "unable to write data - errno=%x\n", errno);
		return 1;
	}
	if (bytesWritten != skDataLen) {
		fprintf(stderr, "wrote only %d bytes instead of %d\n", bytesWritten,
			skDataLen);
		return 1;
	}

	int res = ftruncate(fileDes, skTruncatedLen);
	if (res == -1) {
		fprintf(stderr, "unable to set size of file - errno=%x\n", errno);
		return 1;
	}

	void* pa 
		= mmap(NULL, skTruncatedLen + 1, PROT_READ, MAP_PRIVATE, fileDes, 0);
	if (pa == NULL) {
		fprintf(stderr, "unable to mmap the file - errno=%x\n", errno);
		return 1;
	}
	
	int mappedStrLen = strlen((char*)pa);
	if (mappedStrLen != skTruncatedLen) {
		fprintf(stderr, "mapped string length is %d bytes instead of %d\n", 
			mappedStrLen, skTruncatedLen);
		return 1;
	}

	return 0;
}
